@@ -5,30 +5,26 @@
 
 """Module implementing a remote object allowing easy access to git remotes."""
 
+__all__ = ["RemoteProgress", "PushInfo", "FetchInfo", "Remote"]
+
+import contextlib
 import logging
 import re
 
-from git.cmd import handle_process_output, Git
+from git.cmd import Git, handle_process_output
 from git.compat import defenc, force_text
+from git.config import GitConfigParser, SectionConstraint, cp
 from git.exc import GitCommandError
+from git.refs import Head, Reference, RemoteReference, SymbolicReference, TagReference
 from git.util import (
-    LazyMixin,
-    IterableObj,
-    IterableList,
-    RemoteProgress,
     CallableRemoteProgress,
-)
-from git.util import (
+    IterableList,
+    IterableObj,
+    LazyMixin,
+    RemoteProgress,
     join_path,
 )
 
-from git.config import (
-    GitConfigParser,
-    SectionConstraint,
-    cp,
-)
-from git.refs import Head, Reference, RemoteReference, SymbolicReference, TagReference
-
 # typing-------------------------------------------------------
 
 from typing import (
@@ -47,29 +43,18 @@ from typing import (
     overload,
 )
 
-from git.types import PathLike, Literal, Commit_ish
+from git.types import AnyGitObject, Literal, PathLike
 
 if TYPE_CHECKING:
-    from git.repo.base import Repo
+    from git.objects.commit import Commit
     from git.objects.submodule.base import UpdateProgress
-
-    # from git.objects.commit import Commit
-    # from git.objects import Blob, Tree, TagObject
+    from git.repo.base import Repo
 
 flagKeyLiteral = Literal[" ", "!", "+", "-", "*", "=", "t", "?"]
 
-# def is_flagKeyLiteral(inp: str) -> TypeGuard[flagKeyLiteral]:
-#     return inp in [' ', '!', '+', '-', '=', '*', 't', '?']
-
-
 # -------------------------------------------------------------
 
-
-log = logging.getLogger("git.remote")
-log.addHandler(logging.NullHandler())
-
-
-__all__ = ("RemoteProgress", "PushInfo", "FetchInfo", "Remote")
+_logger = logging.getLogger(__name__)
 
 # { Utilities
 
@@ -79,13 +64,15 @@ def add_progress(
     git: Git,
     progress: Union[RemoteProgress, "UpdateProgress", Callable[..., RemoteProgress], None],
 ) -> Any:
-    """Add the --progress flag to the given kwargs dict if supported by the
-    git command.
+    """Add the ``--progress`` flag to the given `kwargs` dict if supported by the git
+    command.
 
-    :note: If the actual progress in the given progress instance is not
-        given, we do not request any progress.
+    :note:
+        If the actual progress in the given progress instance is not given, we do not
+        request any progress.
 
-    :return: possibly altered kwargs
+    :return:
+        Possibly altered `kwargs`
     """
     if progress is not None:
         v = git.version_info[:2]
@@ -100,24 +87,22 @@ def add_progress(
 
 
 @overload
-def to_progress_instance(progress: None) -> RemoteProgress:
-    ...
+def to_progress_instance(progress: None) -> RemoteProgress: ...
 
 
 @overload
-def to_progress_instance(progress: Callable[..., Any]) -> CallableRemoteProgress:
-    ...
+def to_progress_instance(progress: Callable[..., Any]) -> CallableRemoteProgress: ...
 
 
 @overload
-def to_progress_instance(progress: RemoteProgress) -> RemoteProgress:
-    ...
+def to_progress_instance(progress: RemoteProgress) -> RemoteProgress: ...
 
 
 def to_progress_instance(
-    progress: Union[Callable[..., Any], RemoteProgress, None]
+    progress: Union[Callable[..., Any], RemoteProgress, None],
 ) -> Union[RemoteProgress, CallableRemoteProgress]:
-    """Given the 'progress' return a suitable object derived from RemoteProgress."""
+    """Given the `progress` return a suitable object derived from
+    :class:`~git.util.RemoteProgress`."""
     # New API only needs progress as a function.
     if callable(progress):
         return CallableRemoteProgress(progress)
@@ -202,7 +187,7 @@ class PushInfo(IterableObj):
         self.summary = summary
 
     @property
-    def old_commit(self) -> Union[str, SymbolicReference, Commit_ish, None]:
+    def old_commit(self) -> Union["Commit", None]:
         return self._old_commit_sha and self._remote.repo.commit(self._old_commit_sha) or None
 
     @property
@@ -228,8 +213,8 @@ class PushInfo(IterableObj):
 
     @classmethod
     def _from_line(cls, remote: "Remote", line: str) -> "PushInfo":
-        """Create a new PushInfo instance as parsed from line which is expected to be like
-        refs/heads/master:refs/heads/master 05d2687..1d0568e as bytes."""
+        """Create a new :class:`PushInfo` instance as parsed from line which is expected
+        to be like refs/heads/master:refs/heads/master 05d2687..1d0568e as bytes."""
         control_character, from_to, summary = line.split("\t", 3)
         flags = 0
 
@@ -285,7 +270,7 @@ class PushInfo(IterableObj):
 
 
 class PushInfoList(IterableList[PushInfo]):
-    """IterableList of PushInfo objects."""
+    """:class:`~git.util.IterableList` of :class:`PushInfo` objects."""
 
     def __new__(cls) -> "PushInfoList":
         return cast(PushInfoList, IterableList.__new__(cls, "push_infos"))
@@ -331,7 +316,7 @@ class FetchInfo(IterableObj):
         ERROR,
     ) = [1 << x for x in range(8)]
 
-    _re_fetch_result = re.compile(r"^\s*(.) (\[[\w\s\.$@]+\]|[\w\.$@]+)\s+(.+) -> ([^\s]+)(    \(.*\)?$)?")
+    _re_fetch_result = re.compile(r"^ *(.) (\[[\w \.$@]+\]|[\w\.$@]+) +(.+) -> ([^ ]+)(    \(.*\)?$)?")
 
     _flag_map: Dict[flagKeyLiteral, int] = {
         "!": ERROR,
@@ -344,19 +329,18 @@ class FetchInfo(IterableObj):
 
     @classmethod
     def refresh(cls) -> Literal[True]:
-        """This gets called by the refresh function (see the top level __init__)."""
-        # clear the old values in _flag_map
-        try:
+        """Update information about which :manpage:`git-fetch(1)` flags are supported
+        by the git executable being used.
+
+        Called by the :func:`git.refresh` function in the top level ``__init__``.
+        """
+        # Clear the old values in _flag_map.
+        with contextlib.suppress(KeyError):
             del cls._flag_map["t"]
-        except KeyError:
-            pass
-
-        try:
+        with contextlib.suppress(KeyError):
             del cls._flag_map["-"]
-        except KeyError:
-            pass
 
-        # set the value given the git version
+        # Set the value given the git version.
         if Git().version_info[:2] >= (2, 10):
             cls._flag_map["t"] = cls.TAG_UPDATE
         else:
@@ -369,7 +353,7 @@ class FetchInfo(IterableObj):
         ref: SymbolicReference,
         flags: int,
         note: str = "",
-        old_commit: Union[Commit_ish, None] = None,
+        old_commit: Union[AnyGitObject, None] = None,
         remote_ref_path: Optional[PathLike] = None,
     ) -> None:
         """Initialize a new instance."""
@@ -388,28 +372,31 @@ class FetchInfo(IterableObj):
         return self.ref.name
 
     @property
-    def commit(self) -> Commit_ish:
+    def commit(self) -> "Commit":
         """:return: Commit of our remote ref"""
         return self.ref.commit
 
     @classmethod
     def _from_line(cls, repo: "Repo", line: str, fetch_line: str) -> "FetchInfo":
-        """Parse information from the given line as returned by git-fetch -v
-        and return a new FetchInfo object representing this information.
+        """Parse information from the given line as returned by ``git-fetch -v`` and
+        return a new :class:`FetchInfo` object representing this information.
 
-        We can handle a line as follows:
-        "%c %-\\*s %-\\*s -> %s%s"
+        We can handle a line as follows::
 
-        Where c is either ' ', !, +, -, \\*, or =
-        ! means error
-        + means success forcing update
-        - means a tag was updated
-        * means birth of new branch or tag
-        = means the head was up to date ( and not moved )
-        ' ' means a fast-forward
+            %c %-*s %-*s -> %s%s
 
-        fetch line is the corresponding line from FETCH_HEAD, like
-        acb0fa8b94ef421ad60c8507b634759a472cd56c    not-for-merge   branch '0.1.7RC' of /tmp/tmpya0vairemote_repo
+        Where ``c`` is either a space, ``!``, ``+``, ``-``, ``*``, or ``=``:
+
+        - '!' means error
+        - '+' means success forcing update
+        - '-' means a tag was updated
+        - '*' means birth of new branch or tag
+        - '=' means the head was up to date (and not moved)
+        - ' ' means a fast-forward
+
+        `fetch_line` is the corresponding line from FETCH_HEAD, like::
+
+            acb0fa8b94ef421ad60c8507b634759a472cd56c    not-for-merge   branch '0.1.7RC' of /tmp/tmpya0vairemote_repo
         """
         match = cls._re_fetch_result.match(line)
         if match is None:
@@ -424,7 +411,6 @@ class FetchInfo(IterableObj):
             remote_local_ref_str,
             note,
         ) = match.groups()
-        # assert is_flagKeyLiteral(control_character), f"{control_character}"
         control_character = cast(flagKeyLiteral, control_character)
         try:
             _new_hex_sha, _fetch_operation, fetch_note = fetch_line.split("\t")
@@ -442,7 +428,7 @@ class FetchInfo(IterableObj):
 
         # Parse operation string for more info.
         # This makes no sense for symbolic refs, but we parse it anyway.
-        old_commit: Union[Commit_ish, None] = None
+        old_commit: Union[AnyGitObject, None] = None
         is_tag_operation = False
         if "rejected" in operation:
             flags |= cls.REJECTED
@@ -469,15 +455,17 @@ class FetchInfo(IterableObj):
         if remote_local_ref_str == "FETCH_HEAD":
             ref_type = SymbolicReference
         elif ref_type_name == "tag" or is_tag_operation:
-            # The ref_type_name can be branch, whereas we are still seeing a tag operation.
-            # It happens during testing, which is based on actual git operations.
+            # The ref_type_name can be branch, whereas we are still seeing a tag
+            # operation. It happens during testing, which is based on actual git
+            # operations.
             ref_type = TagReference
         elif ref_type_name in ("remote-tracking", "branch"):
-            # Note: remote-tracking is just the first part of the 'remote-tracking branch' token.
-            # We don't parse it correctly, but its enough to know what to do, and it's new in git 1.7something.
+            # Note: remote-tracking is just the first part of the
+            # 'remote-tracking branch' token. We don't parse it correctly, but it's
+            # enough to know what to do, and it's new in git 1.7something.
             ref_type = RemoteReference
         elif "/" in ref_type_name:
-            # If the fetch spec look something like this '+refs/pull/*:refs/heads/pull/*',
+            # If the fetch spec look something like '+refs/pull/*:refs/heads/pull/*',
             # and is thus pretty much anything the user wants, we will have trouble
             # determining what's going on. For now, we assume the local ref is a Head.
             ref_type = Head
@@ -489,17 +477,19 @@ class FetchInfo(IterableObj):
         if ref_type is SymbolicReference:
             remote_local_ref = ref_type(repo, "FETCH_HEAD")
         else:
-            # Determine prefix. Tags are usually pulled into refs/tags, they may have subdirectories.
-            # It is not clear sometimes where exactly the item is, unless we have an absolute path as
-            # indicated by the 'ref/' prefix. Otherwise even a tag could be in refs/remotes, which is
-            # when it will have the 'tags/' subdirectory in its path.
-            # We don't want to test for actual existence, but try to figure everything out analytically.
+            # Determine prefix. Tags are usually pulled into refs/tags; they may have
+            # subdirectories. It is not clear sometimes where exactly the item is,
+            # unless we have an absolute path as indicated by the 'ref/' prefix.
+            # Otherwise even a tag could be in refs/remotes, which is when it will have
+            # the 'tags/' subdirectory in its path. We don't want to test for actual
+            # existence, but try to figure everything out analytically.
             ref_path: Optional[PathLike] = None
             remote_local_ref_str = remote_local_ref_str.strip()
 
             if remote_local_ref_str.startswith(Reference._common_path_default + "/"):
-                # Always use actual type if we get absolute paths.
-                # Will always be the case if something is fetched outside of refs/remotes (if its not a tag).
+                # Always use actual type if we get absolute paths. This will always be
+                # the case if something is fetched outside of refs/remotes (if its not a
+                # tag).
                 ref_path = remote_local_ref_str
                 if ref_type is not TagReference and not remote_local_ref_str.startswith(
                     RemoteReference._common_path_default + "/"
@@ -513,8 +503,8 @@ class FetchInfo(IterableObj):
                 ref_path = join_path(ref_type._common_path_default, remote_local_ref_str)
             # END obtain refpath
 
-            # Even though the path could be within the git conventions, we make
-            # sure we respect whatever the user wanted, and disabled path checking.
+            # Even though the path could be within the git conventions, we make sure we
+            # respect whatever the user wanted, and disabled path checking.
             remote_local_ref = ref_type(repo, ref_path, check_path=False)
         # END create ref instance
 
@@ -531,10 +521,11 @@ class Remote(LazyMixin, IterableObj):
     """Provides easy read and write access to a git remote.
 
     Everything not part of this interface is considered an option for the current
-    remote, allowing constructs like remote.pushurl to query the pushurl.
+    remote, allowing constructs like ``remote.pushurl`` to query the pushurl.
 
-    :note: When querying configuration, the configuration accessor will be cached
-        to speed up subsequent accesses.
+    :note:
+        When querying configuration, the configuration accessor will be cached to speed
+        up subsequent accesses.
     """
 
     __slots__ = ("repo", "name", "_config_reader")
@@ -558,24 +549,29 @@ class Remote(LazyMixin, IterableObj):
         "--exec",
     ]
 
+    url: str  # Obtained dynamically from _config_reader. See __getattr__ below.
+    """The URL configured for the remote."""
+
     def __init__(self, repo: "Repo", name: str) -> None:
         """Initialize a remote instance.
 
-        :param repo: The repository we are a remote of
-        :param name: The name of the remote, e.g. 'origin'
+        :param repo:
+            The repository we are a remote of.
+
+        :param name:
+            The name of the remote, e.g. ``origin``.
         """
         self.repo = repo
         self.name = name
-        self.url: str
 
     def __getattr__(self, attr: str) -> Any:
-        """Allows to call this instance like
-        remote.special( \\*args, \\*\\*kwargs) to call git-remote special self.name."""
+        """Allows to call this instance like ``remote.special(*args, **kwargs)`` to
+        call ``git remote special self.name``."""
         if attr == "_config_reader":
             return super().__getattr__(attr)
 
-        # Sometimes, probably due to a bug in Python itself, we are being called
-        # even though a slot of the same name exists.
+        # Sometimes, probably due to a bug in Python itself, we are being called even
+        # though a slot of the same name exists.
         try:
             return self._config_reader.get(attr)
         except cp.NoOptionError:
@@ -589,7 +585,10 @@ class Remote(LazyMixin, IterableObj):
         if attr == "_config_reader":
             # NOTE: This is cached as __getattr__ is overridden to return remote config
             # values implicitly, such as in print(r.pushurl).
-            self._config_reader = SectionConstraint(self.repo.config_reader("repository"), self._config_section_name())
+            self._config_reader = SectionConstraint(
+                self.repo.config_reader("repository"),
+                self._config_section_name(),
+            )
         else:
             super()._set_cache_(attr)
 
@@ -610,7 +609,8 @@ class Remote(LazyMixin, IterableObj):
 
     def exists(self) -> bool:
         """
-        :return: True if this is a valid, existing remote.
+        :return:
+            ``True`` if this is a valid, existing remote.
             Valid remotes have an entry in the repository's configuration.
         """
         try:
@@ -624,7 +624,7 @@ class Remote(LazyMixin, IterableObj):
 
     @classmethod
     def iter_items(cls, repo: "Repo", *args: Any, **kwargs: Any) -> Iterator["Remote"]:
-        """:return: Iterator yielding Remote objects of the given repository"""
+        """:return: Iterator yielding :class:`Remote` objects of the given repository"""
         for section in repo.config_reader("repository").sections():
             if not section.startswith("remote "):
                 continue
@@ -638,14 +638,21 @@ class Remote(LazyMixin, IterableObj):
     def set_url(
         self, new_url: str, old_url: Optional[str] = None, allow_unsafe_protocols: bool = False, **kwargs: Any
     ) -> "Remote":
-        """Configure URLs on current remote (cf command git remote set_url).
+        """Configure URLs on current remote (cf. command ``git remote set-url``).
 
         This command manages URLs on the remote.
 
-        :param new_url: String being the URL to add as an extra remote URL
-        :param old_url: When set, replaces this URL with new_url for the remote
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext
-        :return: self
+        :param new_url:
+            String being the URL to add as an extra remote URL.
+
+        :param old_url:
+            When set, replaces this URL with `new_url` for the remote.
+
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
+
+        :return:
+            self
         """
         if not allow_unsafe_protocols:
             Git.check_unsafe_protocols(new_url)
@@ -658,25 +665,33 @@ class Remote(LazyMixin, IterableObj):
         return self
 
     def add_url(self, url: str, allow_unsafe_protocols: bool = False, **kwargs: Any) -> "Remote":
-        """Adds a new url on current remote (special case of git remote set_url).
+        """Adds a new url on current remote (special case of ``git remote set-url``).
 
         This command adds new URLs to a given remote, making it possible to have
         multiple URLs for a single remote.
 
-        :param url: String being the URL to add as an extra remote URL
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext
-        :return: self
+        :param url:
+            String being the URL to add as an extra remote URL.
+
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
+
+        :return:
+            self
         """
         return self.set_url(url, add=True, allow_unsafe_protocols=allow_unsafe_protocols)
 
     def delete_url(self, url: str, **kwargs: Any) -> "Remote":
-        """Deletes a new url on current remote (special case of git remote set_url)
+        """Deletes a new url on current remote (special case of ``git remote set-url``).
 
         This command deletes new URLs to a given remote, making it possible to have
         multiple URLs for a single remote.
 
-        :param url: String being the URL to delete from the remote
-        :return: self
+        :param url:
+            String being the URL to delete from the remote.
+
+        :return:
+            self
         """
         return self.set_url(url, delete=True)
 
@@ -717,9 +732,12 @@ class Remote(LazyMixin, IterableObj):
     def refs(self) -> IterableList[RemoteReference]:
         """
         :return:
-            IterableList of RemoteReference objects. It is prefixed, allowing
-            you to omit the remote path portion, e.g.::
-            remote.refs.master # yields RemoteReference('/refs/remotes/origin/master')
+            :class:`~git.util.IterableList` of :class:`~git.refs.remote.RemoteReference`
+            objects.
+
+            It is prefixed, allowing you to omit the remote path portion, e.g.::
+
+                remote.refs.master  # yields RemoteReference('/refs/remotes/origin/master')
         """
         out_refs: IterableList[RemoteReference] = IterableList(RemoteReference._id_attribute_, "%s/" % self.name)
         out_refs.extend(RemoteReference.list_items(self.repo, remote=self.name))
@@ -729,12 +747,13 @@ class Remote(LazyMixin, IterableObj):
     def stale_refs(self) -> IterableList[Reference]:
         """
         :return:
-            IterableList RemoteReference objects that do not have a corresponding
-            head in the remote reference anymore as they have been deleted on the
-            remote side, but are still available locally.
+            :class:`~git.util.IterableList` of :class:`~git.refs.remote.RemoteReference`
+            objects that do not have a corresponding head in the remote reference
+            anymore as they have been deleted on the remote side, but are still
+            available locally.
 
-            The IterableList is prefixed, hence the 'origin' must be omitted. See
-            'refs' property for an example.
+            The :class:`~git.util.IterableList` is prefixed, hence the 'origin' must be
+            omitted. See :attr:`refs` property for an example.
 
             To make things more complicated, it can be possible for the list to include
             other kinds of references, for example, tag references, if these are stale
@@ -763,13 +782,26 @@ class Remote(LazyMixin, IterableObj):
     def create(cls, repo: "Repo", name: str, url: str, allow_unsafe_protocols: bool = False, **kwargs: Any) -> "Remote":
         """Create a new remote to the given repository.
 
-        :param repo: Repository instance that is to receive the new remote
-        :param name: Desired name of the remote
-        :param url: URL which corresponds to the remote's name
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext
-        :param kwargs: Additional arguments to be passed to the git-remote add command
-        :return: New Remote instance
-        :raise GitCommandError: in case an origin with that name already exists
+        :param repo:
+            Repository instance that is to receive the new remote.
+
+        :param name:
+            Desired name of the remote.
+
+        :param url:
+            URL which corresponds to the remote's name.
+
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
+
+        :param kwargs:
+            Additional arguments to be passed to the ``git remote add`` command.
+
+        :return:
+            New :class:`Remote` instance
+
+        :raise git.exc.GitCommandError:
+            In case an origin with that name already exists.
         """
         scmd = "add"
         kwargs["insert_kwargs_after"] = scmd
@@ -788,7 +820,8 @@ class Remote(LazyMixin, IterableObj):
     def remove(cls, repo: "Repo", name: str) -> str:
         """Remove the remote with the given name.
 
-        :return: The passed remote name to remove
+        :return:
+            The passed remote name to remove
         """
         repo.git.remote("rm", name)
         if isinstance(name, cls):
@@ -799,9 +832,10 @@ class Remote(LazyMixin, IterableObj):
     rm = remove
 
     def rename(self, new_name: str) -> "Remote":
-        """Rename self to the given new_name.
+        """Rename self to the given `new_name`.
 
-        :return: self
+        :return:
+            self
         """
         if self.name == new_name:
             return self
@@ -813,12 +847,15 @@ class Remote(LazyMixin, IterableObj):
         return self
 
     def update(self, **kwargs: Any) -> "Remote":
-        """Fetch all changes for this remote, including new branches which will
-        be forced in (in case your local remote branch is not part the new remote
-        branch's ancestry anymore).
+        """Fetch all changes for this remote, including new branches which will be
+        forced in (in case your local remote branch is not part the new remote branch's
+        ancestry anymore).
 
-        :param kwargs: Additional arguments passed to git-remote update
-        :return: self
+        :param kwargs:
+            Additional arguments passed to ``git remote update``.
+
+        :return:
+            self
         """
         scmd = "update"
         kwargs["insert_kwargs_after"] = scmd
@@ -850,14 +887,14 @@ class Remote(LazyMixin, IterableObj):
             None,
             progress_handler,
             finalizer=None,
-            decode_streams=False,
+            decode_streams=True,
             kill_after_timeout=kill_after_timeout,
         )
 
         stderr_text = progress.error_lines and "\n".join(progress.error_lines) or ""
         proc.wait(stderr=stderr_text)
         if stderr_text:
-            log.warning("Error lines received while fetching: %s", stderr_text)
+            _logger.warning("Error lines received while fetching: %s", stderr_text)
 
         for line in progress.other_lines:
             line = force_text(line)
@@ -878,9 +915,9 @@ class Remote(LazyMixin, IterableObj):
             msg += "length of progress lines %i should be equal to lines in FETCH_HEAD file %i\n"
             msg += "Will ignore extra progress lines or fetch head lines."
             msg %= (l_fil, l_fhi)
-            log.debug(msg)
-            log.debug(b"info lines: " + str(fetch_info_lines).encode("UTF-8"))
-            log.debug(b"head info: " + str(fetch_head_info).encode("UTF-8"))
+            _logger.debug(msg)
+            _logger.debug(b"info lines: " + str(fetch_info_lines).encode("UTF-8"))
+            _logger.debug(b"head info: " + str(fetch_head_info).encode("UTF-8"))
             if l_fil < l_fhi:
                 fetch_head_info = fetch_head_info[:l_fil]
             else:
@@ -892,8 +929,8 @@ class Remote(LazyMixin, IterableObj):
             try:
                 output.append(FetchInfo._from_line(self.repo, err_line, fetch_line))
             except ValueError as exc:
-                log.debug("Caught error while parsing line: %s", exc)
-                log.warning("Git informed while fetching: %s", err_line.strip())
+                _logger.debug("Caught error while parsing line: %s", exc)
+                _logger.warning("Git informed while fetching: %s", err_line.strip())
         return output
 
     def _get_push_info(
@@ -935,7 +972,7 @@ class Remote(LazyMixin, IterableObj):
             if not output:
                 raise
             elif stderr_text:
-                log.warning("Error lines received while fetching: %s", stderr_text)
+                _logger.warning("Error lines received while fetching: %s", stderr_text)
                 output.error = e
 
         return output
@@ -973,31 +1010,36 @@ class Remote(LazyMixin, IterableObj):
             "grab the master branch head from the $URL and store it as my origin
             branch head". And ``git push $URL refs/heads/master:refs/heads/to-upstream``
             means "publish my master branch head as to-upstream branch at $URL".
-            See also git-push(1).
+            See also :manpage:`git-push(1)`.
 
-            Taken from the git manual, gitglossary(7).
+            Taken from the git manual, :manpage:`gitglossary(7)`.
 
-            Fetch supports multiple refspecs (as the
-            underlying git-fetch does) - supplying a list rather than a string
-            for 'refspec' will make use of this facility.
+            Fetch supports multiple refspecs (as the underlying :manpage:`git-fetch(1)`
+            does) - supplying a list rather than a string for 'refspec' will make use of
+            this facility.
 
-        :param progress: See :meth:`push` method.
+        :param progress:
+            See the :meth:`push` method.
 
-        :param verbose: Boolean for verbose output.
+        :param verbose:
+            Boolean for verbose output.
 
         :param kill_after_timeout:
             To specify a timeout in seconds for the git command, after which the process
-            should be killed. It is set to None by default.
+            should be killed. It is set to ``None`` by default.
 
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext.
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
 
-        :param allow_unsafe_options: Allow unsafe options to be used, like --upload-pack.
+        :param allow_unsafe_options:
+            Allow unsafe options to be used, like ``--upload-pack``.
 
-        :param kwargs: Additional arguments to be passed to git-fetch.
+        :param kwargs:
+            Additional arguments to be passed to :manpage:`git-fetch(1)`.
 
         :return:
-            IterableList(FetchInfo, ...) list of FetchInfo instances providing detailed
-            information about the fetch results
+            IterableList(FetchInfo, ...) list of :class:`FetchInfo` instances providing
+            detailed information about the fetch results
 
         :note:
             As fetch does not provide progress information to non-ttys, we cannot make
@@ -1022,7 +1064,7 @@ class Remote(LazyMixin, IterableObj):
             Git.check_unsafe_options(options=list(kwargs.keys()), unsafe_options=self.unsafe_git_fetch_options)
 
         proc = self.repo.git.fetch(
-            "--", self, *args, as_process=True, with_stdout=False, universal_newlines=True, v=verbose, **kwargs
+            "--", self, *args, as_process=True, with_stdout=False, universal_newlines=False, v=verbose, **kwargs
         )
         res = self._get_fetch_info_from_stderr(proc, progress, kill_after_timeout=kill_after_timeout)
         if hasattr(self.repo.odb, "update_cache"):
@@ -1038,16 +1080,29 @@ class Remote(LazyMixin, IterableObj):
         allow_unsafe_options: bool = False,
         **kwargs: Any,
     ) -> IterableList[FetchInfo]:
-        """Pull changes from the given branch, being the same as a fetch followed
-        by a merge of branch with your local branch.
+        """Pull changes from the given branch, being the same as a fetch followed by a
+        merge of branch with your local branch.
 
-        :param refspec: See :meth:`fetch` method
-        :param progress: See :meth:`push` method
-        :param kill_after_timeout: See :meth:`fetch` method
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext
-        :param allow_unsafe_options: Allow unsafe options to be used, like --upload-pack
-        :param kwargs: Additional arguments to be passed to git-pull
-        :return: Please see :meth:`fetch` method
+        :param refspec:
+            See :meth:`fetch` method.
+
+        :param progress:
+            See :meth:`push` method.
+
+        :param kill_after_timeout:
+            See :meth:`fetch` method.
+
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
+
+        :param allow_unsafe_options:
+            Allow unsafe options to be used, like ``--upload-pack``.
+
+        :param kwargs:
+            Additional arguments to be passed to :manpage:`git-pull(1)`.
+
+        :return:
+            Please see :meth:`fetch` method.
         """
         if refspec is None:
             # No argument refspec, then ensure the repo's config has a fetch refspec.
@@ -1063,7 +1118,7 @@ class Remote(LazyMixin, IterableObj):
             Git.check_unsafe_options(options=list(kwargs.keys()), unsafe_options=self.unsafe_git_pull_options)
 
         proc = self.repo.git.pull(
-            "--", self, refspec, with_stdout=False, as_process=True, universal_newlines=True, v=True, **kwargs
+            "--", self, refspec, with_stdout=False, as_process=True, universal_newlines=False, v=True, **kwargs
         )
         res = self._get_fetch_info_from_stderr(proc, progress, kill_after_timeout=kill_after_timeout)
         if hasattr(self.repo.odb, "update_cache"):
@@ -1081,39 +1136,47 @@ class Remote(LazyMixin, IterableObj):
     ) -> PushInfoList:
         """Push changes from source branch in refspec to target branch in refspec.
 
-        :param refspec: See :meth:`fetch` method.
+        :param refspec:
+            See :meth:`fetch` method.
 
         :param progress:
             Can take one of many value types:
 
-            * None to discard progress information.
+            * ``None``, to discard progress information.
             * A function (callable) that is called with the progress information.
               Signature: ``progress(op_code, cur_count, max_count=None, message='')``.
-              `Click here <http://goo.gl/NPa7st>`__ for a description of all arguments
-              given to the function.
-            * An instance of a class derived from :class:`git.RemoteProgress` that
-              overrides the :meth:`~git.RemoteProgress.update` method.
+              See :meth:`RemoteProgress.update <git.util.RemoteProgress.update>` for a
+              description of all arguments given to the function.
+            * An instance of a class derived from :class:`~git.util.RemoteProgress` that
+              overrides the
+              :meth:`RemoteProgress.update <git.util.RemoteProgress.update>` method.
 
-        :note: No further progress information is returned after push returns.
+        :note:
+            No further progress information is returned after push returns.
 
         :param kill_after_timeout:
             To specify a timeout in seconds for the git command, after which the process
-            should be killed. It is set to None by default.
+            should be killed. It is set to ``None`` by default.
 
-        :param allow_unsafe_protocols: Allow unsafe protocols to be used, like ext.
+        :param allow_unsafe_protocols:
+            Allow unsafe protocols to be used, like ``ext``.
 
         :param allow_unsafe_options:
-            Allow unsafe options to be used, like --receive-pack.
+            Allow unsafe options to be used, like ``--receive-pack``.
 
-        :param kwargs: Additional arguments to be passed to git-push.
+        :param kwargs:
+            Additional arguments to be passed to :manpage:`git-push(1)`.
 
         :return:
             A :class:`PushInfoList` object, where each list member represents an
             individual head which had been updated on the remote side.
+
             If the push contains rejected heads, these will have the
-            :attr:`PushInfo.ERROR` bit set in their flags.
-            If the operation fails completely, the length of the returned PushInfoList
-            will be 0.
+            :const:`PushInfo.ERROR` bit set in their flags.
+
+            If the operation fails completely, the length of the returned
+            :class:`PushInfoList` will be 0.
+
             Call :meth:`~PushInfoList.raise_if_error` on the returned object to raise on
             any failure.
         """
@@ -1143,8 +1206,9 @@ class Remote(LazyMixin, IterableObj):
     def config_reader(self) -> SectionConstraint[GitConfigParser]:
         """
         :return:
-            GitConfigParser compatible object able to read options for only our remote.
-            Hence you may simple type config.get("pushurl") to obtain the information.
+            :class:`~git.config.GitConfigParser` compatible object able to read options
+            for only our remote. Hence you may simply type ``config.get("pushurl")`` to
+            obtain the information.
         """
         return self._config_reader
 
@@ -1158,7 +1222,9 @@ class Remote(LazyMixin, IterableObj):
     @property
     def config_writer(self) -> SectionConstraint:
         """
-        :return: GitConfigParser compatible object able to write options for this remote.
+        :return:
+            :class:`~git.config.GitConfigParser`-compatible object able to write options
+            for this remote.
 
         :note:
             You can only own one writer at a time - delete it to release the
